
dpMcmc <- function
(tsDat,  # The time series data, as a list of data frames.
 clustInit,   # Initial cluster assignments, as a vector of the same
             # length as tsDat.
 firstPrior = rep(1, 3), 
 transPrior = matrix(1, nrow=3, ncol=3),  
 alpha = 1.0,   # the precision parameter of the DP
 temp = 1.0,     # the temperatures at which to perform the sampling
 nIter = 10000,  # the number of iterations
 nBurnin = 1000,
 nSave = min( nIter, 1000 ),
 outFile = "dp",
 sampAux = F,
 printStats = T
 ){

  ### Save call information to log file.
  
  nCrises <- length( tsDat )
  nMetrics <- dim( tsDat[[1]] )[2]
  crisLen <- rep( 0, nCrises )
  for( i in 1:nCrises )
    crisLen[i] <- dim( tsDat[[i]] )[1]

  MyInfo <- list()
  MyInfo$Call = deparse(match.call());  #  Get arguments from invocation
  MyInfo$Args = args(dpMcmc)     #  Default values of other args
  MyInfo$Data = tsDat          #  Remember the data
  MyInfo$nCrises = nCrises
  MyInfo$nMetrics = nMetrics
  MyInfo$crisLen = crisLen
  MyInfo$temp = temp
  MyInfo$alpha = alpha
  MyInfo$firstPrior = firstPrior
  MyInfo$transPrior = transPrior

  # Print this information to file
  outFileLG = paste(outFile,"Out.LG",sep="");   # Output Log (dput/dget)
  outFileMC = paste(outFile,"Out.MC",sep="");   # Output from MCMC
  outFileAX = paste(outFile,"Out.AX",sep="");   # Samples of aux params
  unlink(c("outFileMC","outFileLG"));
  dput(MyInfo,outFileLG);

  # Change transPrior to a vector in order to pass to C code (cannot
  # pass it as a matrix)

  transPriorVec <- rep( -1, 9 )
  currentLoc <- 1
  for( i in 1:3 ){
    for( j in 1:3 ){
      transPriorVec[currentLoc] <- transPrior[i,j]
      currentLoc = currentLoc + 1
    }
  }

  # Change tsDat to a vector in order to pass to C code (cannot
  # pass it as a list of data frames)

  crisVec <- rep( -1, sum( crisLen * nMetrics ) )
  currentLoc <- 1
  for( i in 1:nCrises ){
    for( j in 1:crisLen[i] ){
      for( k in 1:nMetrics ){
        crisVec[currentLoc] <- tsDat[[i]][j,k]
        currentLoc = currentLoc + 1
      }
    }
  }
  
  a <- .C( "dpMCMC", crisisDat = as.double(crisVec),
     nCrises = as.integer(nCrises),
     crisLen = as.double(crisLen),
     nMetrics = as.integer(nMetrics),
     typeInit = as.double(clustInit),
     firstPrior = as.double(firstPrior),
     transPrior = as.double(transPriorVec),
     alpha = as.double(alpha),
     temp = as.double(temp),
     nTemps = as.integer( length(temp) ),
     nIter = as.integer(nIter),
     nBurnin = as.integer(nBurnin),
     nSave = as.integer(nSave),
     logFile = outFileMC,
     auxParamFile = outFileAX,
     sampAux = as.integer(sampAux),
     printStats = as.integer(printStats),
     COPY = rep(FALSE, 11) )
  return()
}
