
clustChooseK <- function( dat,
                         firstPrior = c(0,0,0),
                         transPrior = matrix(0, nrow = 3, ncol = 3) ){

  nCris = length(dat)
  nMetr = dim( dat[[1]] )[2]
  crisLen <- rep( 0, nCris )
  for( i in 1:nCris )
    crisLen[i] <- dim( dat[[i]] )[1]

  # Change transPrior to a vector in order to pass to C code (cannot
  # pass it as a matrix)

  transPriorVec <- rep( -1, 9 )
  currentLoc <- 1
  for( i in 1:3 ){
    for( j in 1:3 ){
      transPriorVec[currentLoc] <- transPrior[i,j]
      currentLoc = currentLoc + 1
    }
  }

  # Change dat to a vector in order to pass to C code (cannot
  # pass it as a list of data frames)

  crisVec <- rep( -1, sum( crisLen * nMetr ) )
  currentLoc <- 1
  for( i in 1:nCris ){
    for( j in 1:crisLen[i] ){
      for( k in 1:nMetr ){
        crisVec[currentLoc] <- dat[[i]][j,k]
        currentLoc = currentLoc + 1
      }
    }
  }

  # Create objects in which to return the cluster specs
  bestClust = rep(0, nCris * nCris)  # this will be the matrix
                                     # of assignment probabilities
                                     # (nCris x nClust)
  bestNclust = 0

  res = .C( "clustChooseK",
     dat = as.double(crisVec),
     nCris = as.integer(nCris),
     crisLen = as.double(crisLen),
     nMetr = as.integer(nMetr),
     firstPrior = as.double(firstPrior),
     transPrior = as.double(transPriorVec),
     bestNclust = as.integer(bestNclust),
     bestClust = as.double(bestClust),
     COPY = c( rep(F, 6), rep(T,2) ) )

  nClust = res$bestNclust

  # read res$bestClust into a nCris x nClust
  # matrix
  clustAssign = matrix( 0, ncol = nClust,
    nrow = nCris )
  ind = 1
  for( i in 1:nCris )
    for( j in 1:nClust ){
      clustAssign[i,j] = res$bestClust[ind]
      ind = ind + 1
    }

  return( clustAssign )
}

clustChooseK2 <- function( dat,
                         priorCountGam = c(0,0,0),
                         priorCountT = matrix(0, nrow = 3, ncol = 3) ){
  # Apply the clustering approach outlined in
  # Fraley and Raftery (2002).  First, apply
  # model-based hierarchical clustering,
  # which yields a hierarchy of cluster assignments;
  # these are the rows of initClust.
  initClust = hierarClust( dat, priorCountGam, priorCountT )

  # For each choice of K, apply EM updating
  # and evaluate the resulting BIC.  Store the
  # final probabilistic cluster assignment for each
  # choice of K.
  nChoices = dim(initClust)[1]
  crit = rep(NA, nChoices )
  clustAssign = list()
  for( i in (1:nChoices) ){
    # apply EM updating of the cluster assigment,
    # where the # of clusters is fixed.
    res = emClust( dat, initClust[i,], priorCountGam,
      priorCountT )
    
    # store the probabilistic clustering from EM in the
    # list of cluster assignments.
    clustAssign = c( clustAssign, list( res$probAssign ) )

    # Calculate the BIC criterion for this cluster assignment
    logLike = res$logLike
    nClust = length( unique( initClust[i,] ) )
    crit[i] = crisBIC( logLike, nClust = nClust, nDat = length(dat) )
    print( paste("BIC: ", crit[i] ) )
  }
  # choose the K with highest BIC
  maxCrit = which( crit == max(crit) )[1]
  print( paste("Max BIC: ", max( crit ) ) )

  # return the probabilistic cluster assignment for this K
  return( clustAssign[[maxCrit]] )
}



emClust <- function( dat, clusts, priorCountGam = c(0,0,0),
                    priorCountT = matrix(0, nrow = 3, ncol = 3) ){
  # EM for clustering.  Requires an initial
  # cluster assignment, given in the vector
  # clusts.  Iterates until the improvement in
  # log-likelihood is less than .01, i.e.
  # until the likelihood changes by less than a
  # factor of 1%

  nDat = length(dat)

  # order the cluster labels in the clusts vector,
  # and index without any skips.
  ordClusts = orderTypeVec( clusts )

  # find the # of clusters
  clustIndic = unique(ordClusts)
  nClust = length(clustIndic)

  # initialize the matrix of probabilistic assigments
  # to clusters.
  datClustPr = matrix(0.0, ncol=nClust, nrow = nDat)
  for( i in 1:nDat )
    datClustPr[i, ordClusts[i] ] = 1

  oldLogLike = -10^10  
  logLikeImp = 10^10
  while( logLikeImp > .01 ){
    # while the log-likelihood continues to improve
print(datClustPr)
    # Find the probabilities of the clusters (M-step)
    clustProb = colSums( datClustPr )
    # If we are using priors for the parameters, also add a count
    # of one to each cluster count
    if( any( priorCountGam > 0 ) || any( priorCountT > 0 ) )
      clustProb = clustProb + rep(1, nClust)
    clustProb = clustProb / sum(clustProb)  # normalize the cluster
                                            # prob. vector
    if( any( clustProb < .01 ) )
      print( paste( "Cluster", which( clustProb < .01 ), "has prob. <.01") )

    datClustPrNew = matrix(0, ncol=nClust, nrow = nDat)
    for( j in 1:nClust ){
      # Find the MLEs for the cluster parameters, using the
      # probabilistic cluster assignments (M-step)
      thisClustParams = maxLikeParams( dat, datClustPr[,j],
        priorCountGam, priorCountT )

      # Do the calculations to get the new probabilistic
      # cluster assignments (E-step)
      for( i in 1:nDat ){
        datClustPrNew[i,j] = clustProb[j] *
          evalLike( dat[[i]], gam = thisClustParams$gamHat,
                   Tmat = thisClustParams$tHat,
                   log = F )
      }
    }
    
    # calculate the new mixture log-likelihood and normalize the
    # probabilistic cluster assignments, which are assigned to
    # datClustPr
    newLogLike = 0
    for( i in 1:nDat ){
      # for each observation
      # update the log likelihood
      newLogLike = newLogLike + log( sum( datClustPrNew[i,] ) )
      
      # normalize the cluster probabilities
      datClustPr[i,] = datClustPrNew[i,] / sum(datClustPrNew[i,])
    }
    
    # find the change in log-likelihood
    logLikeImp = newLogLike - oldLogLike
    print(paste("Log likelihood improvement in one EM iteration: ", logLikeImp) )

    # save the log-likelihood
    oldLogLike = newLogLike
  }

  # return the maximized mixture log-likelihood and the probabilistic
  # cluster assignment
  return( list( logLike = newLogLike, probAssign = datClustPr ) )
}

orderTypeVec <- function( clusts ){
  # take a vector with cluster indicators, not necessarily in order
  # and perhaps skipping values; create a new cluster vector with
  # ordered clusters and without skipped values.

  clustLabels = unique( clusts );
  nCris = length(clusts)
  orderedTypes = rep( 0, nCris );
  for( i in 1:nCris ){
    orderedTypes[i] = which( clustLabels == clusts[i] )[1];
  }	
  return( orderedTypes );

}

hierarClust <- function( dat, priorCountGam, priorCountT ){
  #  Model-based hierarchical agglomerative clustering
  #  for the crisis time series model.  Get the sequence
  #  of cluster assignments.
  
  nDat = length( dat )

  # initially assign each observation to its own cluster
  clustAssign = (1:nDat)

  # save the sequence of cluster assignments
  histClusts = matrix(0, ncol = nDat, nrow = 1)
  histClusts[1,] = clustAssign

  # until there is a single cluster
  while( length( unique( clustAssign ) ) > 1 ){

    # find the merge that gives the smallest decrease in the
    # likelihood.  Save the cluster labels in bestPair.
    maxLR = -10^10
    print( "Current cluster assignment: " )
    print( clustAssign )
    
    for( i in unique(clustAssign) )
      for( j in unique(clustAssign) )
        if( i < j ){
          # for each pair of existing clusters
          # calculate the change in the classification likelihood
          # associated with merging the clusters
          lr = likeRatio( dat, clustAssign, i, j, priorCountGam,
            priorCountT )
          print( paste( "Log like. ratio for merging clusters ",
                       i, " and ", j, " is ", lr ) );
          if( lr > maxLR ){
            bestPair = c(i,j)
            maxLR = lr
          }
        }
    
    # merge the clusters
    i=bestPair[1]
    j=bestPair[2]
    clustAssign[ clustAssign == j ] = i
    
    # store the new cluster assignment
    histClusts = rbind( histClusts, clustAssign )
  }

  # return the cluster assignments
  rownames(histClusts) = NULL
  return (histClusts)
}


likeRatio <- function( dat, clustAssign, i, j, priorCountGam,
                      priorCountT ){
  # calculate the log likelihood ratio for merging clusters i and j

  eitherClust = (clustAssign == i) | (clustAssign == j)
  res = maxLogLike( dat[ eitherClust ], priorCountGam,
    priorCountT )
  res = res - maxLogLike( dat[ clustAssign == i ], priorCountGam,
    priorCountT )
  res = res - maxLogLike( dat[ clustAssign == j ], priorCountGam,
    priorCountT )

  return( res )
}

maxLogLike <- function( dat, priorCountGam,
    priorCountT ){
  # given the data from a single cluster, find the maximum log-likelihood

  nMetr = dim( dat[[1]] )[2]
  nDat = length(dat)

  # calculate the MLE for gamma
  yInit = matrix(0, nrow = 3, ncol = nMetr )
  gamHat = matrix(0, nrow = 3, ncol = nMetr )

  for( j in (1:nMetr) ){
    # for each metric j

    # count the number of occurrences of each value of t in Y_{i1j}
    for( i in (1:nDat) ){
      thisY = dat[[i]][1,j] + 1
      yInit[thisY,j] = yInit[thisY,j] + 1
    }
    # normalize the counts to get the MLE for
    # gamma
    gamHat[,j] = yInit[,j] + priorCountGam
    gamHat[,j] = gamHat[,j] / sum(gamHat[,j])
  }

  
  # calculate the MLE for T
  nTrans = array(0, dim= c(nMetr,3,3) )
  tHat = array(0, dim= c(nMetr,3,3) )
  for( j in (1:nMetr) ){
    # for each metric

    # count the number of occurrences of each transition
    for( i in (1:nDat) ){
      crisLen = dim( dat[[i]] )[1]
      for( k in (1:(crisLen-1)) ){
        curVal = dat[[i]][k,j]+1
        newVal = dat[[i]][k+1,j]+1
        nTrans[j,curVal,newVal] = nTrans[j,curVal,newVal] + 1
      }
    }
    # normalize the counts to get the MLE for each row of T
    for( s in (1:3) ){
      if( sum( nTrans[j,s,] ) > 0 ){
        tHat[j,s,] = nTrans[j,s,] + priorCountT[s,]
        if( sum( tHat[j,s,] ) > 0 )
          tHat[j,s,] = tHat[j,s,] / sum( tHat[j,s,] )
      }
    }
  }

  # now calculate the log-likelihood
  logLike = 0
  for( j in (1:nMetr) ){
    # for each metric
    
    # update the log-likelihood
    for( t in (1:3) ){
      if( gamHat[t,j] > 0 )
        logLike = logLike + yInit[t,j] * log( gamHat[t,j] )
      for( s in (1:3) ){
        if( tHat[j,s,t] > 0 )
          logLike = logLike + nTrans[j,s,t] * log( tHat[j,s,t] )
      }
    }
  }
  return(logLike)
}


maxLikeParams <- function( dat, probAssign, priorCountGam = c(0,0,0),
                    priorCountT = matrix(0, nrow = 3, ncol = 3) ){
  # given the data and a vector of probabilities that
  # each observation belongs to this cluster, find the cluster
  # parameters that maximize the log-likelihood (if the prior prob.

  nMetr = dim( dat[[1]] )[2]
  nDat = length(dat)

  # calculate the MLE for gamma
  yInit = matrix( rep(priorCountGam, nMetr),
    nrow = 3, ncol = nMetr, byrow = F )
  gamHat = matrix(0, nrow = 3, ncol = nMetr )

  for( j in (1:nMetr) ){
    # for each metric
    
    # count the number of occurrences of each value of t in Y_{i1j}
    for( i in (1:nDat) ){
      thisY = dat[[i]][1,j] + 1
      yInit[thisY,j] = yInit[thisY,j] + probAssign[i]
    }
    # normalize the counts to get the MLE for gamma
    if( sum(yInit[,j]) > 0 )
      gamHat[,j] = yInit[,j] / sum(yInit[,j])
  }

  # calculate the MLE for T
  nTrans = array(0, dim= c(nMetr,3,3) )
  # initialize with the prior counts
  for( j in (1:nMetr) ){
    nTrans[j,,] = priorCountT
  }
  tHat = array(0, dim= c(nMetr,3,3) )
  for( j in (1:nMetr) ){
    # for each metric
    # count the number of occurrences of each transition
    for( i in (1:nDat) ){
      crisLen = dim( dat[[i]] )[1]
      for( k in (1:(crisLen-1)) ){
        curVal = dat[[i]][k,j]+1
        newVal = dat[[i]][k+1,j]+1
        nTrans[j,curVal,newVal] = nTrans[j,curVal,newVal] +
          probAssign[i]
      }
    }
    # normalize the transition counts to get the MLE for
    # each row of T
    for( s in (1:3) ){
      if( sum( nTrans[j,s,] ) > 0 )
        tHat[j,s,] = nTrans[j,s,] / sum( nTrans[j,s,] )
    }
  }

  return( list( gamHat = gamHat, tHat = tHat ) )
}

evalLike <- function( oneObs, gam, Tmat, logVal=T ){
  # given the data from a single observation and the parameter values,
  # find the log-likelihood (if logVal = T) or the likelihood
  # (otherwise).
  nMetr = dim( oneObs )[2]
  crisLen = dim( oneObs )[1]

  logLike = 0
  for( j in (1:nMetr) ){
    # for each metric

    # calculate the contribution of the first
    # value of the metric, Y_{i1j}
    # to the log-likelihood
    thisY = oneObs[1,j] + 1
    logLike = logLike + log( gam[ thisY,j ] )

    for( k in (1:(crisLen-1)) ){
      # for each transition in the time series

      # calculate the contribution of the transition
      # to the log-likelihood
      curVal = oneObs[k,j]+1
      newVal = oneObs[k+1,j]+1
      logLike = logLike + log( Tmat[j,curVal, newVal] )
    }
  }

  if( logVal )  return( logLike )
  else  return( exp( logLike ) )
}


crisBIC <- function( logLike, nClust, nDat ){
  # Calculate the BIC criterion from the mixture log-likelihood,
  # the # of clusters and the # of observations

  bic = - 8*nClust   # the # of free parameters to be estimated in this model
  bic = bic * log( nDat )
  bic = bic + 2*logLike
  
  return( bic )
}
