\name{simulateTS}
\alias{simulateTS}
\title{Simulate Time Series Data from a Distributed Computing Center}
\description{Simulate time series data according to the alternative
  model described in Woodard and Goldszmidt (2010).}
\usage{
simulateTS( crisTransProb = 0.003, crisLenShape = 3000,
  crisLenRate = 100, crisProbs = c(0.2, 0.45, 0.35),
  nMetrics = 8, nKPIs = 2, nAffectRate = 0.8, nEventRate = 0.4,
  kernMagShape = 60, kernMagRate = 100, meanMu = -2,
  meanTau2 = 0.05, autocorrShape1 = 100,
  autocorrShape2 = 20, arVarShape = 300,
  arVarRate = 10, kernScale = 6,
  tsLength = 4000 )
}
\arguments{
  \item{crisTransProb}{The probability of moving into a crisis period,
    given that you are in a non-crisis period.}
  \item{crisLenShape}{The shape parameter of the negative binomial prior for the
    length of a crisis period (the shape parameter of the underlying
    gamma-distributed parameter)}
  \item{crisLenRate}{The rate parameter of the neg. binomial prior for the
    length of a crisis period (the rate parameter of the underlying
    gamma-distributed parameter)}
  \item{crisProbs}{A vector of probabilities of the crisis types}
  \item{nMetrics}{The number of metrics}
  \item{nKPIs}{The number of key performance indicators}
  \item{nAffectRate}{The rate parameter for the Poisson prior on the
  number of metrics affected by crises of a particular
  type}
  \item{nEventRate}{The rate parameter for the Poisson prior on the
  number of events per time period in the affected metrics during crisis
  periods}
  \item{kernMagShape}{The shape parameter for the gamma-distributed
    kernel magnitudes.}
  \item{kernMagRate}{The rate parameter for the gamma-distributed
    kernel magnitudes.}
  \item{meanMu}{The prior expectation of the mean parameters for the
AR-1 processes of the
  metrics during background periods. These AR-1 processes
  are defined by a mean, a lag-1 autocorrelation, and
  an incremental variance.  These are given
  normal, beta, and inverse gamma prior distributions
  respectively}
  \item{meanTau2}{The prior variance of the mean parameters for the
AR-1 processes of the
metrics during background periods}
  \item{autocorrShape1}{The first shape parameter for the beta prior
    distribution assigned to the autocorrelation parameters of the
AR-1 processes of the
metrics during background periods}
  \item{autocorrShape2}{The second shape parameter for the beta prior
    distribution assigned to the autocorrelation parameters of the
AR-1 processes of the
metrics during background periods}
  \item{arVarShape}{The shape parameter for the gamma prior
    distribution assigned to the incremental variance parameter of the
AR-1 processes of the
metrics during background periods}
  \item{arVarRate}{The rate parameter for the gamma prior
    distribution assigned to the incremental variance parameter of the
AR-1 processes of the
metrics during background periods}
  \item{kernScale}{The kernel scale}  
  \item{tsLength}{The length of the time series}
}
\examples{

metricsTS <- simulateTS()

# Plot the entire multivariate time series 
par(mfrow = c(5,2))
par(mar = c(3.5,5,3,2) + 0.1)
for( k in 1:10 ){
  plot(metricsTS$dat[,k], type = "l",
       ylab = colnames(metricsTS$dat)[k],
       xlab = "", cex.axis = 0.8, cex.lab = 1)
}

# Plot shorter periods in the time series
# to zoom in on any crises that may have
# occurred
par(mfrow = c(5,2))
par(mar = c(3.5,5,3,2) + 0.1)
for( k in 1:10 ){
  plot(metricsTS$dat[1:400,k], type = "l",
       ylab = colnames(metricsTS$dat)[k],
       xlab = "", cex.axis = 0.8, cex.lab = 1)
}

for( k in 1:10 ){
  plot(metricsTS$dat[400:800,k], type = "l",
       ylab = colnames(metricsTS$dat)[k],
       xlab = "", cex.axis = 0.8, cex.lab = 1)
}


}
