#ifndef PRED_CRISIS_H
#define PRED_CRISIS_H


#include "tnt_array1d.h"
#include "tnt_array2d.h"
#include "tnt_array1d_utils.h"
#include "tnt_array2d_utils.h"
#include "moreArray1dUtils.h"
#include "moreArray2dUtils.h"
#include "randGen.h"
#include "timeKeeper.h"
#include <cstdlib>
#include <assert.h>
#include <S.h>

using TNT::Array1D;
using TNT::Array2D;


class PredCrisis{
 private:
  Array2D<double> newDat_,   // The time series data for the new crisis
    *crisisList_,  // The time series data for the previous crises
    transPrior_;
  Array2D<double> **crisisCounts;    // The transition counts for each
				     // metric during each crisis.

  // These count objects are changed for each posterior sample
  Array1D<double> **firstCounts;
  Array2D<double> **transCounts;

  Array1D<double> firstPrior_;

  double likeConst, alpha_;
  char *mcFile_;
  int nCris_, nMetr_, nTemps_;
  bool printInfo_;

  /**************************************************************************
   ** Create a vector of probabilities from a vector of (unnormalized)
   ** log probabilities.
   **/ 
  Array1D<double> toProbs
    ( const Array1D<double> & logProb );

  /**************************************************************************
   ** Calculate the counts for the crisis types and store in firstCounts and 
   ** transCounts.  firstCounts consists of the counts n_ijs where i
   ** is the crisis type, j is the metric, and s is the value of the
   ** first observation in the crisis period.  transCounts is similarly
   ** the count of transitions from s to t for each type and metric.
   **/  
  void calcCounts( const Array1D<double> &crisType ); 

  /**************************************************************************
   ** Calculate the marginal likelihood of a particular vector of
   ** crisis indicators.  crisType should be correctly ordered, such
   ** as the result of a call to orderTypeVec.  Temper the likelihood
   ** by thisTemp.
   **/  
  double logLikelihood( const Array1D<double> &crisType, double thisTemp = 1.0 );

 public:
  PredCrisis(const Array2D<double> &newDat,
	     Array2D<double> *crisisList,
	     char *mcFile,
	     int nCrises, int nMetr,
	     int nTemps, double alpha,
	     const Array1D<double> &firstPrior,
	     const Array2D<double> &transPrior,
	     bool printInfo = true);

  ~PredCrisis();

  /**************************************************************************
   ** Perform prediction.  Return a vector of length # previous crises +
   ** 1, that contains the probability that the new crisis is of the
   ** same type as the previous crises (the last entry is the
   ** probability that the new crisis is of a new type)
   **/  
  Array1D<double> predict();

};

#endif
