
#include "PredCrisisOld.h"
//#include "randGen.h"

/* debugging info: 
   To print high-level debugging info at runtime, add "-DDEBUG1" to the first line of the Makevars
   file. 
*/



#define SQR(x) ((x)*(x))

using TNT::Array1D;
using TNT::Array2D;
using std::cout;
using std::endl;

extern "C"
{
  double S_STDCALL S_rpois( double lambda );
  double S_STDCALL S_rgamma( double s );
  double S_STDCALL S_runif( void );
  double S_STDCALL S_rexp( void );
}


PredCrisisOld::PredCrisisOld
(const Array2D<double> &newDat,
 char *mcFile, char *auxParamFile,
 int nCrises, int nMetr,
 int nTemps, double alpha,
 const Array1D<double> &firstPrior,
 const Array2D<double> &transPrior,
 bool printInfo)
{
  newDat_ = newDat.copy();
  mcFile_ = mcFile;
  auxParamFile_ = auxParamFile;
  nCris_ = nCrises; 
  nMetr_ = nMetr;
  nTemps_ = nTemps;
  alpha_ = alpha;
  firstPrior_ = firstPrior.copy();
  transPrior_ = transPrior.copy();
  printInfo_ = printInfo;

  // calculate the transition counts for the new data
  newCounts = new Array2D<double> [nMetr_];
  int i, k1, k2, crisLen = newDat_.dim1(),
    thisObs, nextObs;
  for( i=0; i<nMetr_; i++ ){
    // for each metric
    //create the counts object
    newCounts[i] = Array2D<double> (3, 3, 0.0);
    for( k1=0; k1<(crisLen-1); k1++ ){
      // for each time period in the crisis except the last	
      // find the transition from time k1 to time k1+1
      thisObs = (int) newDat_[k1][i];
      nextObs = (int) newDat_[k1+1][i];
      // increment the appropriate count
      newCounts [i][thisObs][nextObs] = newCounts [i][thisObs][nextObs] + 1;
    }
    #ifdef DEBUG_PRED
      cout << "new data transition counts for metric " << i << ": " << newCounts[i] << endl;
    #endif
  }

  // calculate a useful constant for marginal likelihood calculations
  likeConst = nMetr_ * lgamma( sum( firstPrior_ ) );  // lgamma is the log-gamma function
  for( k1=0; k1<3; k1++ ){
    likeConst -= nMetr_ * lgamma( firstPrior_[k1] );
    likeConst += nMetr_ * lgamma( transPrior_[k1][0] + transPrior_[k1][1] + transPrior_[k1][2] );
    for( k2=0; k2<3; k2++ ){
      likeConst -= nMetr_ * lgamma( transPrior_[k1][k2] );
    }
  }
  #ifdef DEBUG_PRED
    cout << "Likelihood constant: " << likeConst << endl;
  #endif

}

PredCrisisOld::~PredCrisisOld(){}

/**************************************************************************
 ** Create a vector of probabilities from a vector of (unnormalized)
 ** log probabilities.
 **/ 
Array1D<double> PredCrisisOld::toProbs
( const Array1D<double> & logProb
){
  // subtract the max
  Array1D<double> pVec = logProb - TNT::max( logProb );
  // exponentiate
  pVec = exp( pVec );
  // normalize
  pVec = pVec / TNT::sum(pVec);

  return( pVec );
}


Array1D<double> PredCrisisOld::predict(){
  

  // open files containing MC samples
  FILE *mcFile = fopen(mcFile_, "r" );
  FILE *auxParamFile = fopen(auxParamFile_, "r" );

  int i, j, k1, k2, res;
  Array1D<double> probVec (nCris_+1, 0.0),
    beta(3, 0.0), transRow(3, 0.0),
    typeProb, crisCounts, crisTypes(nCris_, 0.0),
    probSum( nCris_+1, 0.0 );

  int nTypes, nSamps = 0;
  double fromFile, thisVal;
  while( fread( &fromFile, sizeof(double), 1, auxParamFile ) == 1 ){
    // for each posterior sample

    // this is the number of crisis types for 
    // this posterior sample
    nTypes = (int) fromFile;
    #ifdef DEBUG_PRED
      cout << "nTypes: " << nTypes << endl;
    #endif
    // Create a vector to hold the crisis type prob.s
    // (the last element holds the prob. that the new 
    // crisis is a different type)
    typeProb = Array1D<double>( nTypes+1, 0.0 );

    for( i=0; i<nTypes; i++ ){
      // for each crisis type in the historical data
      for( j=0; j<nMetr_; j++ ){
	// for each metric
	// read in beta and update likelihood
	for( k1=0; k1<3; k1++ ){
	  res = fread( &fromFile, sizeof(double), 1, auxParamFile );
	  if( res != 1 )
	    MESSAGE "File ended before content read\n" ERROR;

	  beta[k1] = fromFile;
	  // if the first value of this metric in the new data is k1,
	  // multiply the likelihood by beta[k1]
	  if( (int) newDat_[0][j] == k1 ){
	    typeProb[i] += log( beta[k1] );
	  }
	}
        #ifdef DEBUG_PRED
	cout << "Type: " << i << "  newDat[0][j]: " << newDat_[0][j] << "  beta: " << beta << endl;
        #endif

	// read in T and update likelihood
	for( k1=0; k1<3; k1++ ){
	  for( k2=0; k2<3; k2++ ){
	    // for each possible transition
	    res = fread( &fromFile, sizeof(double), 1, auxParamFile );
	    if( res != 1 )
	      MESSAGE "File ended before content read\n" ERROR;

	    transRow[k2] = fromFile;
	    // multiply the likelihood by this element of the transition
	    // matrix, to the power (# these transitions in new data)
	    typeProb[i] += newCounts[j][k1][k2] * log( transRow[k2] );
	  }
	  #ifdef DEBUG_PRED
  	    cout << "trans counts: " << newCounts[j][k1][0] << " " << newCounts[j][k1][1]  
	       << " " << newCounts[j][k1][2] << "   transRow: " << transRow << endl;
          #endif

	}
	
      }
    }

    // calculate the marginal prob. of a new type (times alpha)
    typeProb[nTypes] = likeConst + log(alpha_);
    for( j=0; j<nMetr_; j++ ){
      // for each metric
      typeProb[nTypes] -= lgamma( sum( firstPrior_ ) + 1.0 );
      for( k1=0; k1<3; k1++ ){
	// for each possible observed value
	thisVal = firstPrior_[k1];
	if( (int)newDat_[0][j] == k1 )
	  thisVal = thisVal + 1.0;
	typeProb[nTypes] += lgamma( thisVal );

	// Now handle the contribution of the transition counts to the 
	// marginal likelihood
	thisVal = transPrior_[k1][0] + transPrior_[k1][1] + transPrior_[k1][2] +
	  newCounts[j][k1][0] + newCounts[j][k1][1] + newCounts[j][k1][2];
        typeProb[nTypes] -= lgamma( thisVal );

	for( k2=0; k2<3; k2++ ){
	  // for each possible observed transition
	  typeProb[nTypes] += lgamma( transPrior_[k1][k2] + newCounts[j][k1][k2] );
	}
      }
    }
    #ifdef DEBUG_PRED
      cout << "typeProb vector after likelihood calc: " << typeProb << endl;
    #endif

    // read in crisis types and calculate the number of crises of each type
    // Create the counts object.
    crisCounts = Array1D<double> (nTypes, 0.0);
    for( i=0; i<nCris_; i++ ){
      // for each crisis
      // read its type from the file (for the first temperature)
      res = fread( &fromFile, sizeof(double), 1, mcFile );
      if( res != 1 )
	MESSAGE "File ended before content read\n" ERROR;

      crisTypes[i] = fromFile;
      crisCounts[ (int)crisTypes[i] ]++;      
    }
    #ifdef DEBUG_PRED
      cout << "crisis types: " << crisTypes << endl;
      cout << "crisis type counts: " << crisCounts << endl;
    #endif

    // Add log count to typeProb
    for( i=0; i<nTypes; i++ ){
      // for each type
      typeProb[i] += log( crisCounts[i] );
    }
    // normalize typeProb
    typeProb = toProbs(typeProb);

    #ifdef DEBUG_PRED
      cout << "final typeProb vector: " << typeProb << endl;
    #endif

    // for each past crisis, calculate pr(znew = zi)
    for( i=0; i<nCris_; i++ ){
      probVec[i] = typeProb[ (int)crisTypes[i] ];
    }
    probVec[nCris_] = typeProb[nTypes];

    #ifdef DEBUG_PRED
      cout << "probVec: " << probVec << endl;
    #endif

    // read through the end of this sample in the MC file
    // (all the temperatures are saved in this file, while
    // only the first temperature is saved in the aux. file.
    for( j=1; j<nTemps_; j++ ){
      // for each temperature except the first
      for( i=0; i<nCris_; i++ ){
	// for each crisis
	res = fread( &fromFile, sizeof(double), 1, mcFile );
	if( res != 1 )
	  MESSAGE "File ended before content read\n" ERROR;

      }
    }
    probSum = probSum + probVec;
    nSamps++;
  }
  probVec = probSum / (double)nSamps;
  if( printInfo_ ){
    cout << "num. posterior samples: " << nSamps << endl;
  }

  fclose( mcFile );  fclose( auxParamFile );
    
  return( probVec );
}
