// C routine to run hierarchical agglomerative clustering, EM, and 
// choosing # clusters using BIC


#define TNT_BOUNDS_CHECK 
// the above ensures that the array bounds are checked when a TNT array is dereferenced
// or assigned
#include "ClustEM.h"
#include <S.h>

using TNT::Array2D;
using std::cout;
using std::endl;

extern "C"{

void clustChooseK(
	    double *dat,   // the crisis data, as a vector
	    long *nCris,  	 // the number of crises
	    double *crisLen,     // a vector of the length of each crisis, in time periods
	    long *nMetr,	 // the number of metrics
	    double *firstPrior,  // The prior counts for the Dirichlet prior on the dist'n of 
	                         // the first value of each metric during a crisis.
	    double *transPrior,  // The prior counts for the product Dirichlet prior on the 
	                         // transition kernel of each metric during a crisis.
	    long *bestNclust,    // an object for returning the "best" choice of K
	    double *bestClust    // an object for returning the probabilistic cluster assignments
	    )
{

  #ifdef DEBUG1
    printf("nCris = %d, nMetr = %d \n", (int)(*nCris), 
	 (int)(*nMetr));
  #endif

  int i, j, k, thisCrisLen,
    currentLoc = 0,
    nCris_ = (int)(*nCris),
    nMetr_ = (int)(*nMetr);

  // convert the dat vector to a vector of matrices containing 
  // the data for each crisis.
  Array2D <double> *crisisList = new Array2D<double> [nCris_];
  
  for( i = 0; i < nCris_; i++ ){
    crisisList[i] = Array2D<double>( (int)crisLen[i], nMetr_ );
    thisCrisLen = (int)crisLen[i];
    for( j = 0; j < thisCrisLen; j++ ){
      for( k = 0; k < nMetr_; k++ ){
	crisisList[i][j][k] = dat[currentLoc];
	currentLoc++;
      }
    }
  }

  // convert the firstPrior double vector to an Array1D object
  Array1D<double> firstPriorArr (3, firstPrior);

  // convert the transPrior double vector to an Array2D object
  Array2D<double> transPriorArr (3, 3, 0.0);
  int k1, k2;
  i=0;
  for( k1=0; k1<3; k1++ ){
    for( k2=0; k2<3; k2++ ){
      transPriorArr[k1][k2] = transPrior[i];
      i++;
    }
  }
  #ifdef DEBUG1
    cout << "firstPrior: " << firstPriorArr << endl;
    cout << "transPrior: " << transPriorArr << endl;
  #endif

  // create the clustEM object
  ClustEM *thisEM = new ClustEM(crisisList, nCris_, 
				firstPriorArr,
				transPriorArr);

  // fit the clustering model
  thisEM->fitClustModel();

  int nClust_ = thisEM->bestNclust();
  (*bestNclust) = nClust_;
  Array2D<double> bestClustArr = thisEM->bestClust();

  // Convert the cluster assignment array into a vector
  i=0;
  for( k1=0; k1<nCris_; k1++ ){
    for( k2=0; k2<nClust_; k2++ ){
      bestClust[i] = bestClustArr[k1][k2];
      i++;
    }
  }
  #ifdef DEBUG1
    cout << "nClust: " << nClust_ << " and best clustering: " << bestClustArr << endl;
  #endif


  // clear memory
  delete thisEM;
  return;
}
          
}


