// C routine to run the MCMC


#define TNT_BOUNDS_CHECK 
// the above ensures that the array bounds are checked when a TNT array is dereferenced
// or assigned
#include "CrisisMCMC.h"
#include <S.h>

using TNT::Array2D;
using std::cout;
using std::endl;

extern "C"{

void dpMCMC(
	    double *crisisDat,   // the crisis data, as a vector
	    long *nCrises,  	 // the number of crises
	    double *crisLen,     // a vector of the length of each crisis, in time periods
	    long *nMetrics,	 // the number of metrics
	    double *typeInit,    // the initial vector of crisis labels
	    double *firstPrior,  // The prior counts for the Dirichlet prior on the dist'n of 
	                         // the first value of each metric during a crisis.
	    double *transPrior,  // The prior counts for the product Dirichlet prior on the 
	                         // transition kernel of each metric during a crisis.
	    double *alpha,       // the Dirichlet process precision parameter
	    double *temp,	 // the temperature (for tempered sampling)
	    long *nTemps,        // the length of the "temp" vector 
	    long *nIter,         // the number of total iterations for the chain
	    long *nBurnin,       // the length of the burnin
	    long *nSave,         // how many samples to store
	    char **logFile,      // the name of the file in which to store the samples
	    char **auxParamFile, // the name of the file in which to store the samples for beta and T
	    long *sampAux,       // Whether or not to sample the auxiliary parameters (beta, T)
	    long *printStats)    // Whether or not to print summary statistics regarding the Markov chain 
                                 // acceptance rates etc.
{

  #ifdef DEBUG1
    printf("nCrises = %d, nMetrics = %d \n", (int)(*nCrises), 
	 (int)(*nMetrics));
  #endif

  int i, j, k, thisCrisLen,
    currentLoc = 0,
    nCrises_ = (int)(*nCrises),
    nMetrics_ = (int)(*nMetrics);

  // convert the crisisDat vector to a vector of matrices containing 
  // the data for each crisis.
  Array2D <double> *crisisList = new Array2D<double> [nCrises_];
  
  for( i = 0; i < nCrises_; i++ ){
    crisisList[i] = Array2D<double>( (int)crisLen[i], nMetrics_ );
    thisCrisLen = (int)crisLen[i];
    for( j = 0; j < thisCrisLen; j++ ){
      for( k = 0; k < nMetrics_; k++ ){
	crisisList[i][j][k] = crisisDat[currentLoc];
	currentLoc++;
      }
    }
  }

  // convert the firstPrior and temp double vectors to Array1D objects
  Array1D<double> firstPriorArr (3, firstPrior),
    tempArr( (*nTemps), temp );
  // convert the transPrior double vector to an Array2D object
  Array2D<double> transPriorArr (3, 3, 0.0);
  int k1, k2;
  i=0;
  for( k1=0; k1<3; k1++ ){
    for( k2=0; k2<3; k2++ ){
      transPriorArr[k1][k2] = transPrior[i];
      i++;
    }
  }
  #ifdef DEBUG1
    cout << "firstPrior: " << firstPriorArr << endl;
    cout << "transPrior: " << transPriorArr << endl;
  #endif

  // create the CrisisMCMC object
  CrisisMCMC *thisMCMC = new CrisisMCMC(crisisList, nCrises_, 
					typeInit, firstPriorArr,
					transPriorArr,
					(*alpha), tempArr,
					(int)(*nIter),
					(int)(*nBurnin),
					(int)(*nSave),
					(*logFile), (*auxParamFile),
					(bool)(*sampAux),
					(bool)(*printStats));

  // simulate the MCMC
  thisMCMC->simulate();

  // clear memory
  delete thisMCMC;
  return;
}
          
}


