

#ifndef MORE_ARRAY2D_UTILS_H
#define MORE_ARRAY2D_UTILS_H

#include "tnt_array2d.h"
#include <string>
#include <iostream>
//#include "jama_eig.h"


namespace TNT {

/* Multiply an array by a scalar */
  Array2D<double> operator*(const Array2D<double> &A, double b);
  Array2D<double> operator*(double a, const Array2D<double> &B);

/* Divide an array by a scalar */
  Array2D<double> operator/(const Array2D<double> &A, double b);

/* Compare each element of the array A to the constant b.  Equality is
  indicated by 1.0, while inequality is indicated by 0.0 */
 Array2D<double> operator== (const Array2D<double> &A, double b);

/* Matrix transpose */
  Array2D<double> arrayTranspose(const Array2D<double> &A);

/* column concatenation of matrices.  If the arrays do not have the
   same number of rows then returns a null array */
  Array2D<double> cbind (const Array2D<double> &A, const Array2D<double> &B);

/* row concatenation of matrices.  If the arrays do not have the
   same number of columns then returns a null array */
  Array2D<double> rbind (const Array2D<double> &A, const Array2D<double> &B);

/* the nrows x nrows identity matrix */
 Array2D<double> identity (int nrows);

/* the nrows x ncols matrix where every i,i element is 1 and the other
   elements are 0. */
 Array2D<double> identity (int nrows, int ncols);

/* Obtain the column sums of the matrix A. */
 Array1D<double> colSums (const Array2D<double> &A);

/* the nrows x nrows matrix where the i,i element is elems[i] and the other
   elements are 0. */
 Array2D<double> diag (const Array1D<double> &elems);

/* the subarray defined by the specified rows and cols of A */
 Array2D<double> subarray (const Array2D<double> &A, int minRow, int maxRow,
			   int minCol, int maxCol);

/* the subarray defined by removing the row corresponding to
   rowToRemove */
 Array2D<double> removeRow (const Array2D<double> &A, int rowToRemove);

/* the subarray defined by removing the column corresponding to
   colToRemove */
 Array2D<double> removeCol (const Array2D<double> &A, int colToRemove);

/* write an array2D as a R/S-plus style matrix */
void writeArray(Array2D<double> &array, std::string filename);
void writeArray(Array2D<double> &array, std::ostream &out);

/* Read an array from a file or input stream in R/S format */
Array2D <double> readDoubleArray2D(std::string filename);
Array2D <double> readDoubleArray2D(std::istream &in);
Array2D <int> readIntArray2D(std::string filename);
Array2D <int> readIntArray2D(std::istream &in);

/* Multiply a matrix and a vector */
Array1D <double> matmult (const Array2D<double> &A, const Array1D<double> &B);

/* Find the least squares solution of A * X = I.  If A is symmetric,
   then A^-1 should also be symmetric.  Due to numerical rounding, it
   is not.  Symmetry of the inverse can be enforced by specifying
   symmetric = true */
Array2D <double> solve (const Array2D<double> &A, bool symmetric = false);

/* Calculate the sum of all of the elements */
 double sum (const Array2D<double> &A);

} //namespace TNT

#endif
