// C routine to predict the type of a new crisis


#define TNT_BOUNDS_CHECK 
// the above ensures that the array bounds are checked when a TNT array is dereferenced
// or assigned
#include "PredCrisisOld.h"
#include <S.h>

using TNT::Array1D;
using TNT::Array2D;
using std::cout;
using std::endl;

extern "C"{

void predictTypeOld(
	    double *newDat,   // the new data, as a vector
	    long *newLen,	 // the number of time periods in the new data
	    char **mcFile,      // the name of the file containing the MCMC samples
	    char **auxParamFile,      // the name of the file containing the aux. parameter samples
	    long *nCrises,  	 // the number of crises
	    long *nMetrics,	 // the number of metrics
	    long *nTemps,	 // the number of temperatures
	    double *alpha,
	    double *firstPrior,  // The prior counts for the Dirichlet prior on the dist'n of 
	                         // the first value of each metric during a crisis.
	    double *transPrior,  // The prior counts for the product Dirichlet prior on the 
	                         // transition kernel of each metric during a crisis.
	    long *printInfo,    // Whether or not to print information
	    double *probVec)    // the vector to contain the prob. of matching previous crises.
{

  int i, j, currentLoc = 0, 
    nCrises_ = (int)(*nCrises),
    nMetrics_ = (int)(*nMetrics),
    newLen_ = (int)(*newLen);

  // convert the newDat vector to matrices containing 
  // the data for new crisis.
  Array2D <double> newMat = Array2D<double> ( (int)(*newLen), nMetrics_, 0.0 );
  
  for( j = 0; j < nMetrics_; j++ ){
    for( i = 0; i < newLen_; i++ ){
      // newDat was stored in row-major order
      newMat[i][j] = newDat[currentLoc];
      currentLoc++;
    }
  }

  // Convert the prior counts into arrays
  Array1D<double> firstPriorArr (3, 0.0);
  Array2D<double> transPriorArr (3, 3, 0.0);
  for( i=0; i<3; i++ ){
    firstPriorArr[i] = firstPrior[i];
    for( j=0; j<3; j++ ){
      transPriorArr[j][i] = transPrior[i*3+j];
    } 
  }

  #ifdef DEBUG_PRED
    cout << "nCrises = " << nCrises_ << ", nMetrics = " << nMetrics_
	 << ", nTemps = " << (*nTemps) << ", alpha = " << (*alpha) << endl;
    cout << "New data: " << newMat << endl;
    cout << "First instance prior counts: " << firstPriorArr << endl;
    cout << "Transition matrix prior counts: " << transPriorArr << endl;
  #endif

  // create the CrisisMCMC object
  PredCrisisOld *predMC = new PredCrisisOld( newMat, 
				       (*mcFile), (*auxParamFile),
				       nCrises_, nMetrics_,
				       (*nTemps), (*alpha),
				       firstPriorArr, transPriorArr,
				       (bool)(*printInfo));

  // get prediction vector
  Array1D<double> probArr = predMC->predict();

  #ifdef DEBUG_PRED
    cout << "probArr: " << probArr << endl;
  #endif

  for( i=0; i<(nCrises_+1); i++ )
    probVec[i] = probArr[i];

  // clear memory
  delete predMC;
  return;
}
          
}


